@@ -9,30 +9,37 @@ from abc import ABC, abstractmethod
 
 import numpy as np
 import scipy.stats as sts
-import scipy.optimize
+import scipy.optimize, scipy.integrate
 
 from .settings import SHAPE_STRING, LOCATION_STRING, SCALE_STRING, SHAPE2_STRING
 from .params import FunctionParam, ConstantParam, Wrapper
 
 __all__ = ["Distribution", "ParametricDistribution", "WeibullDistribution",
-           "LognormalDistribution", "NormalDistribution", "KernelDensityDistribution",
+           "ExponentiatedWeibullDistribution", "LognormalDistribution",
+           "NormalDistribution", "KernelDensityDistribution",
            "MultivariateDistribution"]
 
 
 class Distribution(ABC):
     """
     Abstract base class for distributions.
-
     """
 
     @abstractmethod
-    def cdf(self, x, rv_values, dependency):
-        """Calculate the cumulative distribution function."""
+    def cdf(self, x, rv_values, dependencies):
+        """Cumulative distribution function."""
 
     @abstractmethod
-    def i_cdf(self, probabilities, rv_values, dependency):
-        """Calculate percent-point function. (inverse cumulative distribution function)"""
+    def pdf(self, x, rv_values, dependencies):
+        """Probability density function."""
 
+    @abstractmethod
+    def i_cdf(self, probabilities, rv_values, dependencies):
+        """Inverse cumulative distribution function."""
+
+    def draw_sample(self, n):
+        probabilities = np.random.rand(n)
+        return self.i_cdf(probabilities)
 
 class ParametricDistribution(Distribution, ABC):
     """
@@ -103,15 +110,25 @@ class ParametricDistribution(Distribution, ABC):
 
         Parameters
         ----------
-        shape : Param,
+        shape : Param or float,
             The shape parameter.
-        loc : Param,
+        loc : Param or float,
             The location parameter.
-        scale : Param,
+        scale : Param or float,
             The scale parameter.
-        shape2: Param
+        shape2: Param or float,
             Second shape parameter (if the distribution has one).
         """
+        # In case a parameter is given as a number, convert it to a ConstantParameter.
+        if isinstance(shape, float) or isinstance(shape, int):
+            shape = ConstantParam(shape)
+        if isinstance(loc, float) or isinstance(loc, int):
+            loc = ConstantParam(loc)
+        if isinstance(scale, float) or isinstance(scale, int):
+            scale = ConstantParam(scale)
+        if isinstance(shape2, float) or isinstance(shape2, int):
+            shape2 = ConstantParam(shape2)
+
         self.shape = shape
         self.loc = loc
         self.scale = scale
@@ -142,6 +159,10 @@ class ParametricDistribution(Distribution, ABC):
     def _scipy_i_cdf(self, probabilities, shape, loc, scale):
         """Overwrite with appropriate i_cdf function from scipy package. """
 
+    @abstractmethod
+    def _scipy_pdf(self, x, shape, loc, scale):
+        """Overwrite with appropriate pdf from scipy package."""
+
     def cdf(self, x, rv_values=None, dependencies=None):
         """
         Calculate the cumulative distribution function.
@@ -149,7 +170,7 @@ class ParametricDistribution(Distribution, ABC):
         Parameters
         ----------
         x : array_like
-            Points at which to calculate the cdf.
+            Points at which the cdf is evaluated.
         rv_values : array_like
             Values of all random variables in variable space in correct order.
             This can be a 1-dimensional array with length equal to the number of
@@ -182,7 +203,7 @@ class ParametricDistribution(Distribution, ABC):
         Parameters
         ----------
         probabilities : array_like
-            Probabilities for which to calculate the i_cdf.
+            Probabilities for which the i_cdf is evaluated.
         rv_values : array_like
             Values of all random variables in variable space in correct order.
             This can be a 1-dimensional array with length equal to the number of
@@ -234,8 +255,8 @@ class ParametricDistribution(Distribution, ABC):
 
         Returns
         -------
-        cdf : ndarray,
-            Porbability densities at x under condition rv_values.
+        f : ndarray,
+            Probability densities at x under condition rv_values.
         """
 
         shape_val, loc_val, scale_val, shape2_val = self._get_parameter_values(rv_values, dependencies)
@@ -341,10 +362,9 @@ class ParametricDistribution(Distribution, ABC):
                                      "less than {}, but was {}"
                                      "".format(param_name, valid["max"], param_value))
 
-        def __str__(self):
-            return  "ParametricDistribution with shape={}, loc={}," \
-                    "scale={}, shape2={}.".format(
-                self.shape, self.loc, self.scale, self.shape2)
+    def __str__(self):
+        return  "{} with shape={}, loc={}, scale={}, shape2={}".format(
+            self.__class__.__name__, self.shape, self.loc, self.scale, self.shape2)
 
     @staticmethod
     def param_name_to_index(param_name):
@@ -415,6 +435,9 @@ class WeibullDistribution(ParametricDistribution):
     def _scipy_i_cdf(self, probabilities, shape, loc, scale):
         return sts.weibull_min.ppf(probabilities, c=shape, loc=loc, scale=scale)
 
+    def _scipy_pdf(self, x, shape, loc, scale):
+        return sts.weibull_min.pdf(x, c=shape, loc=loc, scale=scale)
+
 
 class ExponentiatedWeibullDistribution(ParametricDistribution):
     """
@@ -462,38 +485,41 @@ class ExponentiatedWeibullDistribution(ParametricDistribution):
 
         Parameters
         ----------
-        x : array_like
+        x : float or ndarray of floats
             Position where the PDF should be evaluated.
-        shape: float
+        shape: float or ndarray of floats
             beta in https://arxiv.org/pdf/1911.12835.pdf .
-        loc : float
+        loc : float or ndarray of floats
             The distribution does not have a location parameter.
-        scale : float
+        scale : float or ndarray of floats
             alpha in https://arxiv.org/pdf/1911.12835.pdf .
-        shape2 : float
+        shape2 : float or ndarray of floats
             delta in https://arxiv.org/pdf/1911.12835.pdf .
 
         Returns
         -------
-        f : array_like
+        f : ndarray of floats
             Probability density values.
         """
 
         x = np.array(x)
+        x = x.astype(float) # If x elements are int we cannot use np.nan .
+        x[x<=0] = np.nan  # To avoid warnings with negative and 0-values, use NaN.
+
         # In Matlab syntax: f = delta .* beta ./ alpha .* (x ./ alpha).^
         # (beta - 1) .* (1 - exp(-1 * (x ./ alpha).^beta)).^(delta - 1) .*
         # exp(-1 .* (x ./ alpha).^beta);
-        a = scale
-        b = shape
-        d = shape2
+        a = np.array(scale)
+        b = np.array(shape)
+        d = np.array(shape2)
         term1 = d * np.divide(b, a) * np.power(np.divide(x, a), b - 1)
         term2 = np.power(1 - np.exp(-1 * np.power(np.divide(x, a), b)), d - 1)
         term3 = np.exp(-1 * np.power(np.divide(x, a), b))
         f = np.multiply(term1, np.multiply(term2, term3))
         f = np.array(f) # Ensure that f is an numpy array, also if x is 1D.
 
-        # Ensure that PDF(negative value) = 0
-        f[x < 0] = 0
+        # Ensure that PDF(-inf < x <= 0) = 0
+        f[np.isnan(x)] = 0
 
         return f
 
@@ -607,6 +633,10 @@ class ExponentiatedWeibullDistribution(ParametricDistribution):
             else:
                 err_msg = "Error. At least one parameter needs to be free to fit it."
                 raise NotImplementedError(err_msg)
+        else:
+            raise NotImplementedError("Currently only weighted least squares "
+                                "estimation is implemented.")
+
 
         params = (pHat[1], loc, pHat[0], shape2) # shape, location, scale, shape2
         constantParams = (ConstantParam(pHat[1]), # shape parameter.
@@ -691,6 +721,9 @@ class LognormalDistribution(ParametricDistribution):
     def _scipy_i_cdf(self, probabilities, shape, _, scale):
         return sts.lognorm.ppf(probabilities, s=shape, scale=scale)
 
+    def _scipy_pdf(self, x, shape, loc, scale):
+        return sts.lognorm.pdf(x, s=shape, scale=scale)
+
     def __str__(self):
         if hasattr(self, "mu"):
             return  "LognormalDistribution with shape={}, loc={}," \
@@ -735,10 +768,14 @@ class NormalDistribution(ParametricDistribution):
     def _scipy_i_cdf(self, probabilities, _, loc, scale):
         return sts.norm.ppf(probabilities, loc=loc, scale=scale)
 
+    def _scipy_pdf(self, x, shape, loc, scale):
+        return sts.norm.pdf(x, loc=loc, scale=scale)
+
 
 class MultivariateDistribution():
     """
-    A Multivariate distribution consisting of multiple univariate distributions and dependencies.
+    A multivariate distribution that is structured as a hierarchical model.
+    It consists of of multiple univariate distributions and dependencies.
 
     Attributes
     ----------
@@ -766,6 +803,173 @@ class MultivariateDistribution():
         if not distributions is None:
             self.add_distributions(distributions, dependencies)
 
+    def cdf(self, x, lower_integration_limit=(-np.inf, -np.inf)):
+        """
+        Joint cumulative distribution function.
+
+        Parameters
+        ----------
+        x : 2-dimensional ndarray
+            Points at which the cdf is evaluated.
+            Array is of shape (d, n) with d being the number of variables and
+            n being the number of points.
+        lower_integration_limit : array_like, optional
+            Lower limits for the ingration of the pdf. Must have a length of n.
+            Defaults to (-np.inf, -np.inf).
+        Returns
+        -------
+        p : array_like
+            Probabilities.
+        """
+        x = np.array(x)
+
+        if x.shape[0] == 2:
+            x0 = x[0]
+            x1 = x[1]
+            p = np.empty(x1.size)
+            if x0.size == 1:
+                p, error = scipy.integrate.nquad(self.pdf_2d, [
+                    [lower_integration_limit[0], x0],
+                    [lower_integration_limit[1], x1]
+                ])
+            else:
+                for i in range(x0.size):
+                    p[i], error = scipy.integrate.nquad(self.pdf_2d, [
+                        [lower_integration_limit[0], x0[i]],
+                        [lower_integration_limit[1], x1[i]]
+                    ])
+        else:
+            raise NotImplementedError('CDF is currently only implemented for '
+                                      'two-dimensional joint distributions.')
+
+        return p
+
+    def pdf(self, x):
+        """
+        Joint probability density function.
+
+        Parameters
+        ----------
+        x : 2-dimensional ndarray
+            Points at which the pdf is evaluated.
+            Array is of shape (d, n) with d being the number of variables and
+            n being the number of points.
+
+        Returns
+        -------
+        f : ndarray of floats
+            Probability densities.
+        """
+        x = np.array(x)
+        f = np.empty(x[0].size)
+
+        for i in range(self.n_dim):
+            if i == 0:
+                f = self.distributions[i].pdf(x[i])
+            else:
+                univariate_f = self.distributions[i].pdf(
+                    x=x[i],
+                    rv_values=x[0:i],
+                    dependencies=self.dependencies[i])
+                f = np.multiply(f, univariate_f)
+
+        return f
+
+    def pdf_2d(self, x, y):
+        """
+        Probability density function for a 2D joint distribution.
+
+        x : ndarray
+        y : ndarray
+
+        Returns
+        -------
+        f : ndarray of floats
+            Probability densities.
+        """
+        f = self.pdf([x, y])
+        return f
+
+    def marginal_cdf(self, x, dim=0):
+        """
+        Marginal cumulative distribution function in given dimension.
+
+        Parameters
+        ----------
+        x : ndarray
+        dim : int (optional)
+            Defaults to 0, which means the first dimension. 1 = second dimension.
+
+
+        Returns
+        -------
+        F : ndarray of floats
+            Probabilities.
+        """
+        raise NotImplementedError
+
+    def marginal_icdf(self, p, dim=0):
+        """
+        Marginal inverse cumulative distribution function in given dimension.
+
+        Parameters
+        ----------
+        p : ndarray of floats
+            Probabilities.
+        dim : int (optional)
+            Defaults to 0, which means the first dimension. 1 = second dimension.
+
+        Returns
+        -------
+        x : ndarray of floats
+        """
+        raise NotImplementedError
+
+    def marginal_pdf(self, x, dim=0):
+        """
+        Marginal probability density function in given dimension.
+
+        Parameters
+        ----------
+        x : ndarray
+        dim : int (optional)
+            Defaults to 0, which means the first dimension. 1 = second dimension.
+
+
+        Returns
+        -------
+        f : ndarray of floats
+            Probability densities.
+        """
+
+
+    def draw_sample(self, n):
+        """
+        Parameters
+        ----------
+        n : number of observations that shall be drawn.
+
+        Returns
+        -------
+        sample : 2-dimensional ndarray
+            Array is of shape (d, n) with d being the number of variables and
+            n being the number of observations.
+        """
+        sample = []
+        if len(self.distributions) > 0:
+            sample.append(self.distributions[0].draw_sample(n))
+        i = 1
+        while i < len(self.distributions):
+            # If this dimension is independent the parameters are directly available.
+            if all(d is None for d in self.dependencies[i]):
+                sample.append(self.distributions[0].draw_sample(n))
+            # Otherwise, the conditioning random variables need to be evaluated.
+            else:
+                sample.append(self.distributions[i].i_cdf(
+                    np.random.rand(n), sample, self.dependencies[i]))
+            i = i + 1
+
+        return sample
 
     def add_distributions(self, distributions, dependencies):
         """
@@ -908,10 +1112,10 @@ class MultivariateDistribution():
             for i in range(len(coords)):
                 if i < len(multi_index):
                     current_point[i] = coords[i][multi_index[i]]
-                else:  # random variable must be independent of this dimensions, so set to 0
+                else:  # Random variable must be independent of this dimensions, so set to 0.
                     current_point[i] = 0
 
-            # calculate averaged pdf
+            # Calculate averaged pdf.
             lower = cdf(coords[dist_index] - 0.5 * dx, current_point, dependency)
             upper = cdf(coords[dist_index] + 0.5 * dx, current_point, dependency)
             fbar[f_index] = (upper - lower)  # / dx
@@ -1151,7 +1355,7 @@ class KernelDensityDistribution(Distribution):
         Parameters
         ----------
         x : array_like
-            Points at which to calculate the cdf.
+            Points at which the cdf is evaluated.
         rv_values : array_like
             Values of all random variables in variable space in correct order.
             This can be a 1-dimensional array with length equal to the number of
@@ -1171,15 +1375,41 @@ class KernelDensityDistribution(Distribution):
         """
         result = []
         for point in x:
-            # scale x
+            # Scale x.
             x_point = point * (len(self._cdf) - 1) / (max(self._i_cdf) - min(self._i_cdf))
-            # use linear fit if x_point is between two points
+            # Use linear fit if x_point is between two points.
             linear_fit = np.poly1d(np.polyfit([int(x_point), int(x_point) + 1],
                                               [self._cdf[int(x_point)],
                                                self._cdf[int(x_point) + 1]], 1))
             result.append(linear_fit(x_point))
         return result
 
+    def pdf(self, x, rv_values=None, dependencies=None):
+        """
+        Probability density function.
+
+        Parameters
+        ----------
+        x : array_like
+            Points at which the PDF should be evaluated.
+        rv_values : array_like
+            Values of all random variables in variable space in correct order.
+            This can be a 1-dimensional array with length equal to the number of
+            random variables N or a 2-dimensional array with shape (N, M).
+            If x is an array, M must be len(x).
+        dependencies : tuple
+            A 3-element tuple with one entry each for the shape, loc and scale parameters.
+            The entry is the index of the random variable the parameter depends on.
+            The index order has to be the same as in rv_values.
+
+
+        Returns
+        -------
+        f : ndarray,
+            Probability densities at x under condition rv_values.
+        """
+        raise NotImplementedError
+
     def i_cdf(self, probability, rv_values, dependencies):
         """
         Calculate percent-point function. (inverse cumulative distribution function)
@@ -1187,7 +1417,7 @@ class KernelDensityDistribution(Distribution):
         Parameters
         ----------
         probabilities : array_like
-            Probabilities for which to calculate the i_cdf.
+            Probabilities for which the i_cdf is evaluated.
         rv_values : array_like
             Values of all random variables in variable space in correct order.
             This can be a 1-dimensional array with length equal to the number of
@@ -1207,9 +1437,9 @@ class KernelDensityDistribution(Distribution):
         """
         result = []
         for point in probability:
-            # scale probability
+            # Scale probability.
             x_point = point * (len(self._i_cdf) - 1)
-            # use linear fit if x_point is between two points
+            # Use linear fit if x_point is between two points.
             linear_fit = np.poly1d(np.polyfit([int(x_point), int(x_point) + 1],
                                               [self._i_cdf[int(x_point)],
                                                self._i_cdf[int(x_point) + 1]], 1))
