@@ -480,33 +480,33 @@ class Fit():
     >>> #fig = plt.figure(figsize=(10, 8))
     >>> #example_text = fig.suptitle("Dependence of 'scale'")
     >>>
-    >>> #ax_1 = fig.add_subplot(221)
-    >>> #title1 = ax_1.set_title("Fitted curve")
+    >>> #ax1 = fig.add_subplot(221)
+    >>> #title1 = ax1.set_title("Fitted curve")
     >>> param_grid = my_fit.multiple_fit_inspection_data[1].scale_at
     >>> x_1 = np.linspace(5, 15, 100)
-    >>> #ax1_plot = ax_1.plot(param_grid, my_fit.multiple_fit_inspection_data[0].scale_value, 'x')
-    >>> #example_plot1 = ax_1.plot(x_1, my_fit.mul_var_dist.distributions[1].scale(x_1))
+    >>> #ax1_plot = ax1.plot(param_grid, my_fit.multiple_fit_inspection_data[0].scale_value, 'x')
+    >>> #example_plot1 = ax1.plot(x_1, my_fit.mul_var_dist.distributions[1].scale(x_1))
     >>>
-    >>> #ax_2 = fig.add_subplot(222)
-    >>> #title2 = ax_2.set_title("Distribution '1'")
-    >>> #ax2_hist = ax_2.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[0], normed=1)
+    >>> #ax2 = fig.add_subplot(222)
+    >>> #title2 = ax2.set_title("Distribution '1'")
+    >>> #ax2_hist = ax2.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[0], normed=1)
     >>> shape = my_fit.mul_var_dist.distributions[1].shape(0)
     >>> scale = my_fit.mul_var_dist.distributions[1].scale(param_grid[0])
-    >>> #ax2_plot = ax_2.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
+    >>> #ax2_plot = ax2.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
     >>>
-    >>> #ax_3 = fig.add_subplot(223)
-    >>> #title3 = ax_3.set_title("Distribution '2'")
-    >>> #ax3_hist = ax_3.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[1], normed=1)
+    >>> #ax3 = fig.add_subplot(223)
+    >>> #title3 = ax3.set_title("Distribution '2'")
+    >>> #ax3_hist = ax3.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[1], normed=1)
     >>> shape = my_fit.mul_var_dist.distributions[1].shape(0)
     >>> scale = my_fit.mul_var_dist.distributions[1].scale(param_grid[1])
-    >>> #ax3_plot = ax_3.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
+    >>> #ax3_plot = ax3.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
     >>>
-    >>> #ax_4 = fig.add_subplot(224)
-    >>> #title4 = ax_4.set_title("Distribution '3'")
-    >>> #ax4_hist = ax_4.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[2], normed=1)
+    >>> #ax4 = fig.add_subplot(224)
+    >>> #title4 = ax4.set_title("Distribution '3'")
+    >>> #ax4_hist = ax4.hist(my_fit.multiple_fit_inspection_data[1].scale_samples[2], normed=1)
     >>> shape = my_fit.mul_var_dist.distributions[1].shape(0)
     >>> scale = my_fit.mul_var_dist.distributions[1].scale(param_grid[2])
-    >>> #ax4_plot = ax_4.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
+    >>> #ax4_plot = ax4.plot(np.linspace(0, 20, 100), sts.lognorm.pdf(np.linspace(0, 20, 100), s=shape, scale=scale))
 
     """
 
@@ -607,8 +607,13 @@ class Fit():
         list_number_of_intervals = []
         list_width_of_intervals = []
         for dist_description in dist_descriptions:
+            if dist_description.get(
+                    'number_of_intervals') == None and dist_description.get(
+                    'width_of_intervals') == None:
+                dist_description['number_of_intervals'] = 15
             list_number_of_intervals.append(dist_description.get('number_of_intervals'))
             list_width_of_intervals.append(dist_description.get('width_of_intervals'))
+
         for dist_description in dist_descriptions:
             dist_description['list_number_of_intervals'] = list_number_of_intervals
             dist_description['list_width_of_intervals'] = list_width_of_intervals
