@@ -13,18 +13,18 @@ __all__ = ["plot_sample", "plot_marginal_fit", "plot_dependence_functions",
            "plot_wave_breaking_limit", "hs_from_limiting_sig_wave_steepness"]
 
 
-def plot_sample(plotted_sample, ax=None, do_plot_rasterized=True):
+def plot_sample(sample_plot_data, ax=None, do_plot_rasterized=True):
     """
     Plots the sample of metocean data.
 
     Parameters
     ----------
-    plotted_sample : SamplePlotData,
+    sample_plot_data : SamplePlotData,
         The sample that should be plotted and its meta information.
     """
     if ax is None:
-        ax = plotted_sample.ax
-    ps = plotted_sample
+        ax = sample_plot_data.ax
+    ps = sample_plot_data
     x = ps.x
     y = ps.y
     if ps.x_inside is not None and ps.y_inside is not None:
@@ -42,10 +42,16 @@ def plot_sample(plotted_sample, ax=None, do_plot_rasterized=True):
             ax.scatter(x, y, s=40, alpha=0.5, c='k', marker='.',
                        label='observation', rasterized=do_plot_rasterized)
 
+    # Remove axis on the right and on the top (Matlab 'box off').
+    ax.spines['right'].set_visible(False)
+    ax.spines['top'].set_visible(False)
+    ax.yaxis.set_ticks_position('left')
+    ax.xaxis.set_ticks_position('bottom')
+
 
 def plot_marginal_fit(sample, dist, fig, ax=None, label=None, color_sample='k',
                       marker_sample='x', marker_size_sample=3, color_fit='b',
-                      dataset_char='?',legend_fontsize=8):
+                      dataset_char=None, legend_fontsize=8):
     """
     Plots the fitted marginal distribution versus a dataset in a quantile-
     quantile (QQ) plot.
@@ -87,12 +93,12 @@ def plot_marginal_fit(sample, dist, fig, ax=None, label=None, color_sample='k',
     ax.title.set_text('')
     ax.spines['right'].set_visible(False)
     ax.spines['top'].set_visible(False)
-    if max(sample) < 12:
-        plt.xlim((0, 12))
-        plt.ylim((0, 15.5))
-    else:
-        plt.xlim((0, 35))
-        plt.ylim((0, 35))
+    #if max(sample) < 12:
+        #plt.xlim((0, 12))
+        #plt.ylim((0, 15.5))
+    #else:
+        #plt.xlim((0, 35))
+        #plt.ylim((0, 35))
     if dist.name == 'ExponentiatedWeibull':
         dist_description = 'Exponentiated Weibull\n' \
                            '($\\alpha$=' + str('%.3g' % dist.scale(0)) + ', ' \
@@ -106,15 +112,21 @@ def plot_marginal_fit(sample, dist, fig, ax=None, label=None, color_sample='k',
         dist_description = dist.name
     plt.legend(['Dataset '+ dataset_char, dist_description], loc='upper left',
                frameon=False, prop={'size': legend_fontsize})
-    plt.xlabel('Theoretical quantiles, ' + str(label).lower())
-    plt.ylabel('Ordered values, ' + str(label).lower())
+    xlabel_string = 'Theoretical quantiles'
+    ylabel_string = 'Ordered values'
+    if label:
+        xlabel_string += ', ' + str(label).lower()
+        ylabel_string += ', ' + str(label).lower()
+    plt.xlabel(xlabel_string)
+    plt.ylabel(ylabel_string)
 
 
 def plot_dependence_functions(
         fit, fig, ax1=None, ax2=None, unconditonal_variable_label=None,
         marker_discrete='o', markersize_discrete=5,
         markerfacecolor_discrete='lightgray', markeredgecolor_discrete='k',
-        style_dependence_function='b-', legend_fontsize=8):
+        style_dependence_function='b-', legend_fontsize=8,
+        factor_draw_longer=1.1):
     """
     Plots the fitted dependence function using two subplots, one subplot showing
     the fit of the shape value and one subplot showing the fit of the scale
@@ -136,11 +148,13 @@ def plot_dependence_functions(
     marker_discrete : char, defaults to 'o'
     markersize_discrete : int, defaults to 5
     markerfacecolor_discrete : color (char, string or RGB), defaults to 'lightgray'
-    markeredgecolor_discrete : color (char, string or RGB),d efaults to 'k'
+    markeredgecolor_discrete : color (char, string or RGB), defaults to 'k'
     style_dependence_function : str, defaults to 'b-'
         Style of the fitted dependence function.
     legend_fontsize : int, defaults to 8
         Font size of the legend's text.
+    factor_draw_longer : float, defaults to 1.1
+        How much longer than the last point should the fitted curve be plotted.
 
     Raises
     ------
@@ -163,7 +177,7 @@ def plot_dependence_functions(
 
     plt.sca(ax1)
     scale_at = fit.multiple_fit_inspection_data[1].scale_at
-    x1 = np.linspace(0, max(scale_at)*1.1, 100)
+    x1 = np.linspace(0, max(scale_at) * factor_draw_longer, 100)
     if fit.mul_var_dist.distributions[1].scale.func_name == 'power3':
         dp_function = '$' + str('%.3g' % fit.mul_var_dist.distributions[1].scale.a) + \
                       '+' + str('%.3g' % fit.mul_var_dist.distributions[1].scale.b) + \
@@ -190,8 +204,8 @@ def plot_dependence_functions(
         plt.plot(x1, np.log(fit.mul_var_dist.distributions[1].scale(x1)),
                  style_dependence_function, label=dp_function)
         ylabel = '$μ_{tz}$'
-        plt.xlim((0, 6))
-        plt.ylim((0.9, 2.15))
+        #plt.xlim((0, 6))
+        #plt.ylim((0.9, 2.15))
     if fit.mul_var_dist.distributions[1].name == 'Weibull' or \
                     fit.mul_var_dist.distributions[1].name == 'ExponentiatedWeibull':
         plt.plot(scale_at, fit.multiple_fit_inspection_data[1].scale_value,
@@ -203,8 +217,8 @@ def plot_dependence_functions(
         plt.plot(x1, fit.mul_var_dist.distributions[1].scale(x1),
                  style_dependence_function, label=dp_function)
         ylabel = '$α_{hs}$'
-        plt.xlim((0, 30))
-        plt.ylim((0, 10))
+        #plt.xlim((0, 30))
+        #plt.ylim((0, 10))
     plt.xlabel(unconditonal_variable_label)
     plt.legend(frameon=False, prop={'size': legend_fontsize})
     ax1.spines['right'].set_visible(False)
@@ -213,7 +227,7 @@ def plot_dependence_functions(
 
     plt.sca(ax2)
     shape_at = fit.multiple_fit_inspection_data[1].shape_at
-    x1 = np.linspace(0, max(shape_at)*1.1, 100)
+    x1 = np.linspace(0, max(shape_at) * factor_draw_longer, 100)
     plt.plot(shape_at, fit.multiple_fit_inspection_data[1].shape_value,
              marker_discrete,
              markersize=markersize_discrete,
@@ -223,8 +237,8 @@ def plot_dependence_functions(
              style_dependence_function)
     plt.xlabel(unconditonal_variable_label)
     if fit.mul_var_dist.distributions[1].name == 'Lognormal':
-        plt.xlim((0, 6))
-        plt.ylim((0.065, 0.33))
+        #plt.xlim((0, 6))
+        #plt.ylim((0.065, 0.33))
         ylabel = '$σ_{tz}$'
         if fit.mul_var_dist.distributions[1].shape.func_name == 'exp3':
             dp_function = '$' + str('%.3g' % fit.mul_var_dist.distributions[1].shape.a) + \
@@ -244,10 +258,10 @@ def plot_dependence_functions(
     if fit.mul_var_dist.distributions[1].name == 'Weibull'  or \
                     fit.mul_var_dist.distributions[1].name == 'ExponentiatedWeibull':
         ylabel = '$β_{h_s}$'
-        plt.xlim((0, 30))
-        plt.ylim((0, 3.5))
+        #plt.xlim((0, 30))
+        #plt.ylim((0, 3.5))
         if fit.mul_var_dist.distributions[1].shape.func_name == 'power3':
-            dp_function = '$' + str('%.3g' % fit.mul_var_dist.distributions[1].shape.a) + \
+            dp_function = '$' + str('%.4f' % fit.mul_var_dist.distributions[1].shape.a) + \
                           '+' + str('%.3g' % fit.mul_var_dist.distributions[1].shape.b) + \
                           '\cdot h_s^{' + str('%.3g' % fit.mul_var_dist.distributions[1].shape.c) + '}$'
         elif fit.mul_var_dist.distributions[1].shape.func_name == 'logistics4':
@@ -266,8 +280,8 @@ def plot_dependence_functions(
     plt.ylabel(ylabel)
 
 
-def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
-                 line_style='b-', alpha=1, plotted_sample=None, x_lim = None,
+def plot_contour(x, y, ax=None, contour_label=None, x_label=None, y_label=None,
+                 line_style='b-', alpha=1, sample_plot_data=None, x_lim=None,
                  upper_ylim=None, median_x=None, median_y=None, median_style='r-',
                  median_label='median of x2|x1'):
     """
@@ -281,8 +295,9 @@ def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
         The contour's coordinates in the x-direction.
     y : ndarray of floats
         The contour's coordiantes in the y-direction.
-    ax : Axes
+    ax : Axes, optional (defaults to None)
         Axes of the figure where the contour should be plotted.
+        If None a figure is created here.
     contour_label : str, optional (defaults to None)
         The environmental contour's label that will be used in the legend.
     x_label : str, optional (defaults to None)
@@ -293,7 +308,7 @@ def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
         Matplotlib line style.
     alpha : float, optional (default to 1)
         Alpha value (transparency) for the contour's line.
-    plotted_sample : SamplePlotData, optional (defaults to None)
+    sample_plot_data : SamplePlotData, optional (defaults to None)
         The sample that should be plotted and its meta information.
     x_lim : tuple of floats, optional (defaults to None)
         x-Axis limit.
@@ -309,6 +324,10 @@ def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
         Label for the legend of the plotted median line.
     """
 
+    if ax is None:
+        fig = plt.figure()
+        ax = fig.add_subplot(111)
+
     # For generating a closed contour: add the first coordinate at the end.
     xplot = x.tolist()
     xplot.append(x[0])
@@ -316,8 +335,8 @@ def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
     yplot.append(y[0])
 
     # Plot the contour and, if provided, also the sample.
-    if plotted_sample:
-        plot_sample(plotted_sample, ax=ax)
+    if sample_plot_data:
+        plot_sample(sample_plot_data, ax=ax)
     if contour_label:
         ax.plot(xplot, yplot, line_style, alpha=alpha, label=contour_label)
     else:
@@ -335,14 +354,14 @@ def plot_contour(x, y, ax, contour_label=None, x_label=None, y_label=None,
     if x_lim:
         plt.xlim(x_lim)
     y_lim_factor = 1.2
-    if plotted_sample and upper_ylim is None:
+    if sample_plot_data and upper_ylim is None:
         # If there is not enough space for the legend in the upper left corner:
         # make space for it.
-        max_index = np.where(plotted_sample.y == max(plotted_sample.y))
-        if plotted_sample.x[max_index] < 0.6 * max(max(x), max(plotted_sample.x)):
+        max_index = np.where(sample_plot_data.y == max(sample_plot_data.y))
+        if sample_plot_data.x[max_index] < 0.6 * max(max(x), max(sample_plot_data.x)):
             y_lim_factor = 1.35
 
-        upper_ylim = max(max(y), max(plotted_sample.y)) * y_lim_factor
+        upper_ylim = max(max(y), max(sample_plot_data.y)) * y_lim_factor
     elif upper_ylim is None:
         upper_ylim = max(y) * y_lim_factor
     plt.ylim((0, upper_ylim))
@@ -382,7 +401,7 @@ class SamplePlotData():
         outside datapoints.
     """
 
-    def __init__(self, x, y, ax, label=None, x_inside=None, y_inside=None,
+    def __init__(self, x, y, ax=None, label=None, x_inside=None, y_inside=None,
                  x_outside=None, y_outside=None, return_period=None):
         """
         Parameters
@@ -391,7 +410,7 @@ class SamplePlotData():
             The sample's first environmental variable.
         y : ndarray of floats
             The sample's second environmental variable.
-        ax : Axes
+        ax : Axes, optional (defaults to None)
             Axes of the figure where the scatter plot should be drawn.
         label : str, optional (defaults to None)
             Label that will be used in the legend for the sample.
@@ -420,7 +439,7 @@ class SamplePlotData():
 
 def plot_confidence_interval(x_median, y_median, x_bottom, y_bottom, x_upper,
                              y_upper, ax, x_label=None, y_label=None,
-                             contour_labels=[None, None, None], plotted_sample=None):
+                             contour_labels=[None, None, None], sample_plot_data=None):
     """
     Plots the confidence interval (median, bottom, upper) in a standardized
     appearance.
@@ -446,7 +465,7 @@ def plot_confidence_interval(x_median, y_median, x_bottom, y_bottom, x_upper,
         Label for the y-axis.
     contour_labels : list of str, optional (defaults to [None, None, None])
         Label for the environmental contours that will be used in the legend.
-    plotted_sample : SamplePlotData, optional
+    sample_plot_data : SamplePlotData, optional
         If provided, this sample is plotted together with the contours.
     """
     plot_contour(x=x_median,
@@ -456,7 +475,7 @@ def plot_confidence_interval(x_median, y_median, x_bottom, y_bottom, x_upper,
                  y_label=y_label,
                  line_style='b-',
                  contour_label=contour_labels[0],
-                 plotted_sample=plotted_sample)
+                 sample_plot_data=sample_plot_data)
     plot_contour(x=x_bottom,
                  y=y_bottom,
                  contour_label=contour_labels[1],
